/*
 * moleculer-db-adapter-cassandra
 * Copyright (c) 2019 MoleculerJS (https://github.com/moleculerjs/moleculer-db)
 * MIT Licensed
 */

"use strict";

const { ServiceSchemaError } = require("moleculer").Errors;
const ExpressCassandra = require("express-cassandra");

class CassandraAdapter {

	/**
	 * Creates an instance of CassandraAdapter.
	 * @param {Object?} clientopts
	 * @param {String?} dbName
	 *
	 * @memberof CassandraAdapter
	 */
    constructor(opts, ormOpts) {
        this.clientOpts = opts;
        this.ormOpts = ormOpts;
    }

	/**
	 * Initialize adapter
	 *
	 * @param {ServiceBroker} broker
	 * @param {Service} service
	 *
	 * @memberof CassandraAdapter
	 */
    init(broker, service) {
        this.broker = broker;
        this.service = service;

        if (!this.service.schema.model) {
            /* istanbul ignore next */
            throw new ServiceSchemaError("Missing `model` definition in service!");
        }
        if (!this.service.schema.model.name) {
            /* istanbul ignore next */
            throw new ServiceSchemaError("Missing `name` definition in model of service!");
        }
    }

	/**
	 * Connect to database
	 *
	 * @returns {Promise}
	 *
	 * @memberof CassandraAdapter
	 */
    async connect() {
        this.client = ExpressCassandra.createClient({
            clientOptions: this.clientOpts,
            ormOptions: this.ormOpts,
        });
        const model = this.client.loadSchema(this.service.schema.model.name, this.service.schema.model.schema);
        await model.syncDBAsync();
        this.model = this.client.instance[this.service.schema.model.name];
    }

	/**
	 * Disconnect from database
	 *
	 * @returns {Promise}
	 *
	 * @memberof CassandraAdapter
	 */
    disconnect() {
        return Promise.resolve();
    }

	/**
	 * Find all entities by filters.
	 *
	 * Available filter props:
	 * 	- limit
	 *  - offset
	 *  - sort
	 *  - search
	 *  - searchFields
	 *  - query
	 *
	 * @param {Object} filters
	 * @returns {Promise<Array>}
	 *
	 * @memberof CassandraAdapter
	 */
    find(filters) {
        if (filters.search) {
            throw new Error("Not implemented");
        }
        const params = { ...filters.query };
        if (filters.sort) {
            params.$orderby = filters.sort;
        }
        if (filters.limit) {
            params.$limit = filters.limit;
        }
        if (filters.searchFields) {
            return this.model.findAsync(params, { select: filters.searchFields });
        }
        return this.model.findAsync(params);
    }

	/**
	 * Find an entity by query
	 *
	 * @param {Object} query
	 * @returns {Promise}
	 * @memberof CassandraAdapter
	 */
    findOne(query) {
        return this.model.findOneAsync(query);
    }

	/**
	 * Find an entities by ID.
	 *
	 * @param {String} id
	 * @returns {Promise<Object>} Return with the found document.
	 *
	 * @memberof CassandraAdapter
	 */
    findById(id) {
        return this.model.findOneAsync({ id });
    }

	/**
	 * Find any entities by IDs.
	 *
	 * @param {Array} idList
	 * @returns {Promise<Array>} Return with the found documents in an Array.
	 *
	 * @memberof CassandraAdapter
	 */
    findByIds(idList) {
        return this.model.findAsync({
            id: {
                $in: idList
            }
        });
    }

	/**
	 * Get count of filtered entites.
	 *
	 * Available query props:
	 *  - search
	 *  - searchFields
	 *  - query
	 *
	 * @param {Object} [filters={}]
	 * @returns {Promise<Number>} Return with the count of documents.
	 *
	 * @memberof CassandraAdapter
	 */
    count(filters = {}) {
        throw new Error("Not implemented");
    }

	/**
	 * Insert an entity.
	 *
	 * @param {Object} entity
	 * @returns {Promise<Object>} Return with the inserted document.
	 *
	 * @memberof CassandraAdapter
	 */
    async insert(entity) {
        const obj = new this.model(entity);
        return obj.saveAsync();
    }

	/**
	 * Insert many entities
	 *
	 * @param {Array} entities
	 * @returns {Promise<Array<Object>>} Return with the inserted documents in an Array.
	 *
	 * @memberof CassandraAdapter
	 */
    insertMany(entities) {
        throw new Error("Not implemented");
    }

	/**
	 * Update many entities by `query` and `update`
	 *
	 * @param {Object} query
	 * @param {Object} update
	 * @returns {Promise<Number>} Return with the count of modified documents.
	 *
	 * @memberof CassandraAdapter
	 */
    updateMany(query, update) {
        throw new Error("Not implemented");
    }

	/**
	 * Update an entity by ID and `update`
	 *
	 * @param {String} id
	 * @param {Object} update
	 * @returns {Promise<Object>} Return with the updated document.
	 *
	 * @memberof CassandraAdapter
	 */
    updateById(id, update) {
        throw new Error("Not implemented");
    }

	/**
	 * Remove entities which are matched by `query`
	 *
	 * @param {Object} query
	 * @returns {Promise<Number>} Return with the count of deleted documents.
	 *
	 * @memberof CassandraAdapter
	 */
    removeMany(query) {
        throw new Error("Not implemented");
    }

	/**
	 * Remove an entity by ID
	 *
	 * @param {String} id
	 * @returns {Promise<Object>} Return with the removed document.
	 *
	 * @memberof CassandraAdapter
	 */
    removeById(id) {
        throw new Error("Not implemented");
    }

	/**
	 * Clear all entities from collection
	 *
	 * @returns {Promise}
	 *
	 * @memberof CassandraAdapter
	 */
    clear() {
        throw new Error("Not implemented");
    }

    /**
	 * Convert DB entity to JSON object
	 *
	 * @param {any} entity
	 * @returns {Object}
	 * @memberof CassandraAdapter
	 */
    entityToObject(entity) {
        if (!entity.toJSON) {
            return entity;
        }
        return entity.toJSON();
    }

	/**
	 * Create a filtered cursor.
	 *
	 * Available filters in `params`:
	 *  - search
	 * 	- sort
	 * 	- limit
	 * 	- offset
	 *  - query
	 *
 	 * @param {Object} params
 	 * @param {Boolean} isCounting
	 * @returns {Cursor}
	 */
    createCursor(params, isCounting) {
        throw new Error("Not implemented");
    }

    /**
	* For compatibility only.
	* @param {Object} entity
	* @param {String} idField
	* @memberof CassandraAdapter
	* @returns {Object} Entity
	*/
    beforeSaveTransformID(entity, idField) {
        return entity;
    }

	/**
	* For compatibility only.
	* @param {Object} entity
	* @param {String} idField
	* @memberof CassandraAdapter
	* @returns {Object} Entity
	*/
    afterRetrieveTransformID(entity, idField) {
        return entity;
    }
}

module.exports = CassandraAdapter;
